package com.cweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.*;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.HighOrder;
import com.hai.entity.HighUser;
import com.hai.entity.HighUserCoupon;
import com.hai.entity.HighUserPayPassword;
import com.hai.model.HighUserModel;
import com.hai.model.ResponseData;
import com.hai.service.HighOrderService;
import com.hai.service.HighUserCouponService;
import com.hai.service.HighUserPayPasswordService;
import com.hai.service.HighUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.util.StringUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Auther: 袁野
 * @Description:
 * @Date: 2021/3/30 22:50
 */
@Controller
@RequestMapping(value = "/highUser")
@Api(value = "用户接口")
public class HighUserController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighUserService highUserService;

    @Resource
    private HighUserCouponService highUserCouponService;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private HighUserPayPasswordService highUserPayPasswordService;

    @RequestMapping(value = "/findUser", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询详情")
    public ResponseData findById( HttpServletRequest request) {
        try {
            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            HighUser highUser = highUserService.findByUserId(userInfoModel.getHighUser().getId());
            if (highUser != null) {
                highUser.setUnusedCouponNum( highUserCouponService.getCouponList(highUser.getId(), 1).size()); //未使用卡卷数量
                highUser.setUnpaid(highOrderService.countOrderByUserId(highUser.getId() , 1));
                highUser.setUnusedDiscount(highOrderService.countUnusedDiscountByUserId(highUser.getId() , 1));
                highUser.setIsSetPayPwd(highUserPayPasswordService.isSetPayPwd(userInfoModel.getHighUser().getId()));
            }
            return ResponseMsgUtil.success(highUser);

        } catch (Exception e) {
            log.error("HighUserController --> findByUserId() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


    @RequestMapping(value = "/bindUserPhone", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "绑定用户手机号")
    public ResponseData bindUserPhone(@RequestParam(value = "phone", required = true) String phone,
                                        @RequestParam(value = "code", required = true) String code,
                                        HttpServletRequest request, HttpServletResponse response) {
        try {
            VerifyCode verifyCode = VerifyCodeStorage.getDate(phone);
            if(verifyCode == null) {
                log.error("HighUserController --> bindUserPhone() error!", "验证码错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "验证码错误");
            }
            if(!verifyCode.getObject().equals(code)) {
                log.error("HighUserController --> bindUserPhone() error!", "验证码错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "验证码错误");
            }

            HighUser highUser = highUserService.bindUserPhone(phone, request);

            HighUserModel highUserModel = new HighUserModel();
            highUser.setPassword(null);
            highUserModel.setHighUser(highUser);
            SessionObject so = new SessionObject(highUser.getUnionId(), 1 , highUserModel);
            userCenter.save(request, response, so);
            return ResponseMsgUtil.success(so);

        } catch (Exception e) {
            log.error("HighUserController --> bindUserPhone() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/isSetPayPwd", method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "用户是否设置支付密码")
    public ResponseData isSetPayPwd(HttpServletRequest request) {
        try {
            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();
            return ResponseMsgUtil.success(highUserPayPasswordService.isSetPayPwd(userInfoModel.getHighUser().getId()));

        } catch (Exception e) {
            log.error("HighUserController --> setUserPayPwd() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/setUserPayPwd", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "设置用户支付密码")
    public ResponseData setUserPayPwd(@RequestBody JSONObject body,HttpServletRequest request, HttpServletResponse response) {
        try {
            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();

            if (StringUtils.isBlank(body.getString("password"))
                    || body.getString("password").length() != 6){
                log.error("HighUserController --> setUserPayPwd() error!", "请设置6位支付密码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "请设置6支付密码");
            }

            // 查询用户密码
            HighUserPayPassword userPayPassword = highUserPayPasswordService.getDetailByUser(userInfoModel.getHighUser().getId());
            if (userPayPassword != null) {
                log.error("HighUserController --> setUserPayPwd() error!", "您已设置了支付密码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REPEAT_SET_USER_PAY_PWD, "");
            }

            userPayPassword = new HighUserPayPassword();
            userPayPassword.setUserId(userInfoModel.getHighUser().getId());
            userPayPassword.setPassword(AESEncodeUtil.aesEncrypt(body.getString("password")));
            highUserPayPasswordService.editUserPayPwd(userPayPassword);

            // 定义个人所有数据
            HighUser user = highUserService.findByUserId(userInfoModel.getHighUser().getId());
            HighUserModel highUserModel = new HighUserModel();
            user.setPassword(null);
            user.setIsSetPayPwd(highUserPayPasswordService.isSetPayPwd(user.getId()));
            highUserModel.setHighUser(user);
            SessionObject so = new SessionObject(user.getUnionId(), 1 , highUserModel);
            return ResponseMsgUtil.success(so);

        } catch (Exception e) {
            log.error("HighUserController --> setUserPayPwd() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/updateUserPayPwd", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改用户支付密码")
    public ResponseData updateUserPayPwd(@RequestBody JSONObject body,HttpServletRequest request, HttpServletResponse response) {
        try {
            // 用户
            SessionObject sessionObject = userCenter.getSessionObject(request);
            HighUserModel userInfoModel = (HighUserModel) sessionObject.getObject();
            if (StringUtils.isBlank(body.getString("password"))
                ||  StringUtils.isBlank(body.getString("newPassword"))) {
                log.error("HighUserController --> setUserPayPwd() error!", "");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 查询用户密码
            HighUserPayPassword userPayPassword = highUserPayPasswordService.getDetailByUser(userInfoModel.getHighUser().getId());
            if (userPayPassword == null) {
                log.error("HighUserController --> setUserPayPwd() error!", "修改失败，未设置支付密码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.NOT_SET_USER_PAY_PWD, "");
            }

            if (!AESEncodeUtil.aesEncrypt(body.getString("password")).equals(userPayPassword.getPassword())) {
                log.error("HighUserController --> setUserPayPwd() error!", "修改失败，旧密码不一致");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "修改失败，旧密码不一致");
            }

            userPayPassword.setPassword(AESEncodeUtil.aesEncrypt(body.getString("newPassword")));
            highUserPayPasswordService.editUserPayPwd(userPayPassword);
            return ResponseMsgUtil.success("操作成功");

        } catch (Exception e) {
            log.error("HighUserController --> updateUserPayPwd() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
